import formDialog from '@/components/mixins/formDialog'
import baseLazyTree from '@/components/mixins/baseLazyTree'
import baseTree from '@/components/mixins/baseTree'
import baseHeader from '@/components/mixins/baseHeader'
import { mapGetters, mapActions } from 'vuex'
import { getTreeIds } from '@/utils'

export default {
  name: 'columnTree',
  mixins: [baseHeader, formDialog, baseLazyTree,baseTree],
  data () {
    return {
      tabs: [],
      dataType: 2,
      searchValue: '',
      columnId: null,
      columnData: {},
      currentTab: 'base',
      tree: {
        defaultExpandAll: false,
        expandOnClickNode: false,
        treeName: 'cloumn',
        tableLazy:true,
        expendedKeys: [],
        defaultProps: {
          children: 'children',
          label: 'name'
        }
      },
      addFormLoading: false,
      addForm: {
        modelId: '',
        channelParentId: [],
        channelName: '',
        channelPath: ''
      },
      addFormItems: [{
        prop: 'channelParentId',
        label: '上级栏目',
        type: 'cascader',
        options: 'getColumnOptions'
      },
      {
        prop: 'modelId',
        label: '栏目模型',
        type: 'select',
        options: 'columnModelOptions'
      },
      {
        prop: 'channelName',
        label: '栏目名称',
        maxlength: 15
      },
      {
        prop: 'channelPath',
        label: '访问路径',
        maxlength: 30
      }
      ],
      optionsArr: '',
      ids: ''
    }
  },
  provide () {
    return {
      optionArray: () => this.optionsArr,
      cid: () => this.ids
      // optionArray: 123
    }
  },
  watch: {
    'addForm.modelId': function (newData, oldData) {
      if (newData && newData !== oldData) {
        const that = this
        const form = { ...this.addForm }
        that.$request.fetchChannelPlus(newData).then(res => {
          if (res.code === 200) {
            if (res.data.enableJson && res.data.enableJson.formListBase instanceof Array) {
              res.data.enableJson.formListBase.forEach(d => {
                if (d.value) {
                  let v = d.value
                  let item = that.addFormItems.find(f => f.prop === v.name)
                  if (item) {
                    item.maxlength = v.max || ''
                    item.placeholder = v.placeholder || ''
                    item.explain = v.tip || ''
                  }
                }
              })
            }

            that.$refs.addDialog.$refs.form.resetFields()
            that.addForm = form
          }
        })
      } else {
        this.addFormItems.forEach(item => {
          item.placeholder = ''
          item.explain = ''
        })
      }
    },
    'addForm.channelName': function (newData, oldData) {
      if (newData && newData !== oldData) {
        this.$request.fetchPinyin({ text: newData }).then(r => {
          if (r.code === 200) {
            this.addForm.channelPath = r.data
          }
        })
      } else {
        this.addForm.channelPath = ''
      }
    },
    // workflowOptions(val){
    //   let lists = [...this.workflowOptions.map(item=>item.id)];
    //  this.ids = lists[0];
    // },
    'detail.dataField.workflowId' (val) {
      let lists = [...this.workflowOptions.map(item => item.id)]
      // this.ids = lists[0];
      //  console.log(lists,333333)
      // this.workflowOptions[0].ids = lists[0];
      this.optionsArr = this.workflowOptions
      if (this.optionsArr.length) {
        this.$set(this.optionsArr[0], 'ids', Math.random(1000))
      }
      this.ids = this.detail.dataField.workflowId
    //  console.log(this.ids, this.optionsArr, 33333);
    },
    getdata: {
      handler (val) {
        // console.log('2222222222222', val)
        new Promise((r, j) => {
          r(this.detail)
        }).then((res) => {
          this.ids = res.dataField.workflowId
          this.optionsArr = val
          // console.log(this.ids, this.optionsArr, 222222);
        })
      },
      deep: true
    }

  },
  computed: {
    ...mapGetters(['columnOptions', 'columnModelOptions']),
    ...mapGetters('column', ['detail', 'detailLoading']),
    ...mapGetters(['columnOptions', 'workflowOptions']),
    columnHeaders () {
      return {}
    },
    getTabs () {
      return this.tabs
    },
    getColumnOptions () {
      const loop = function (data) {
        if (data instanceof Array && data.length) {
          return data.map(d => {
            if (!d.createChildAble) {
              d.disabled = false
            }
            if (d.children instanceof Array && d.children.length) {
              loop(d.children)
            }
            return d
          })
        }
      }
      // console.log(this.columnOptions,'this.columnOptions');
      return [{
        label: '无',
        value: ''
      }, ...(loop(this.columnOptions) || [])]
    },
    addRules () {
      return {
        channelParentId: [
          this.$rules.required()
        ],
        modelId: [
          this.$rules.required()
        ],
        channelName: [
          this.$rules.required(),
          // this.$rules.cnEnNumSymbol(), bug -23050 栏目名称还是有限制，完全取消限制
          {
            validator: (rule, value, callback) => {
              if (value) {
                var params = {
                  name: value
                }
                this.$request.fetchChannelUnique(params).then(res => {
                  if (res.code === 200 && res.data) {
                    callback()
                  } else {
                    callback(new Error('栏目名称不能重复'))
                  }
                })
              }
            },
            trigger: 'blur'
          }
        ],
        channelPath: [
          this.$rules.required(),
          this.$rules.enNum(),
          {
            validator: (rule, value, callback) => {
              if (value) {
                var params = {
                  path: value
                }
                this.$request.fetchChannelUnique(params).then(res => {
                  if (res.code === 200 && res.data) {
                    callback()
                  } else {
                    this.$request.fetchChannelCountPathPrefix({ path: value }).then(res => {
                      if (res.code === 200 && res.data) {
                        this.$set(this.addForm, rule.field, value + res.data)
                      }
                    })
                    callback(new Error('栏目路径不能重复'))
                  }
                })
              }
            },
            trigger: 'blur'
          }
        ]
      }
    },
    getdata () {
      return this.$store.state.config.workflowOptions
    }
  },
  methods: {

    ...mapActions('column', ['FetchColumnOptions', 'FetchColumnModelOptions']),
    getTreeIds,
    getColumnDetail () { },
    // 点击站点
    handleClick (data) {
      this.getColumnDetail(data.id)

      if (!this.detail.viewAble || !this._checkPermission('/channel', 'GET')) {
        this.$message.error('该站点没有查看权限')
      }
      this.setColumnId('')
      this.setColumnId(Number(data.id))
      this.columnData = { ...this.detail }
      this.addForm.channelParentId = this.getTreeIds(this.columnOptions, data.id)
    },
    // 搜索
    handleSearch () {
      this.tree.tableLazy = this.searchValue?false:true;
      this.fetchChannelTree()
    },
    // 获取栏目树结构
    fetchChannelTree () {
      // console.log(this.siteBaseConfig,"this.siteBaseConfig");
      this.tree.loading = true
      const params = {
        name: this.searchValue,
        lazy: this.siteBaseConfig.channelTreeLazy ? this.siteBaseConfig.channelTreeLazy : true // 判断是否异步
        // lazy:true
      }
      const sortPerm = this._checkPermission('/channel/sort', 'PUT')
      return this.$request.fetchChannelTree(params).then(res => {
        // console.log(res);
        if (res.code === 200) {
          const loop = function (data) {
            if (data instanceof Array && data.length) {
              return data.map((d, i, arr) => {
                if (i > 0 && d.editAble && sortPerm) d.prevId = arr[i - 1].id
                if (arr.length > i + 1 && d.editAble && sortPerm) d.nextId = arr[i + 1].id

                if (d.children instanceof Array && d.children.length) {
                  loop(d.children)
                }
                return d
              })
            }
          }
          this.tree.data = loop(res.data)
        }
        this.tree.loading = false
        return res
      }).catch(() => {
        this.tree.loading = false
      })
    },
    // 参数过滤
    handleFilterParams (params) {
      const channelParentId = params.channelParentId.length > 0 ? params.channelParentId[params.channelParentId.length - 1] : ''
      params.channelParentId = channelParentId
      return params
    },
    // 新建保存
    handleConfirmCreate (data) {
      let params = this.handleFilterParams(data)
      this.addFormLoading = true
      if (params.channelParentId) {
        this.$request.fetchChannelCreate(params).then(res => {
          this.$restBack(res, () => {
            this.setColumnId(params.channelParentId)
            this.fetchChannelTree()
            this.FetchColumnOptions(true)
            this.FetchContentColumnOptions(true)
          }, '新增成功')
          this.addFormLoading = false
        }).catch(() => {
          this.addFormLoading = false
        })
      } else {
        this.$request.fetchChannelCreateTop(params).then(res => {
          this.$restBack(res, () => {
            // 判断是否空的栏目
            if (this.columnId == null) {
              this.setColumnId(1)
            }
            this.fetchChannelTree()
            this.FetchColumnOptions(true)
            this.FetchContentColumnOptions(true)
          }, '新增成功')
          this.addFormLoading = false
        }).catch(() => {
          this.addFormLoading = false
        })
      }
    },
    initChannelTree () {
      this.fetchChannelTree().then(res => {
        if (res.code === 200 && res.data instanceof Array && res.data.length) {
          // console.log('内容',res.data);
          if (!this.columnId) {
            this.columnData = res.data[0]
            this.setColumnId(res.data[0].id)
            this.getColumnDetail(res.data[0].id)
            this.$request.fetchChannelDetail(res.data[0].id).then(res => {
              if (res.code == 200) {
                this.columnData = res.data
              }
            })
          } else {
            const that = this
            const loop = data => {
              if (data instanceof Array && data.length) {
                data.forEach(d => {
                  if (d.id === this.columnId) {
                    that.columnData = d
                  } else if (d.children) {
                    loop(d.children)
                  }
                })
              }
            }
            loop(res.data)
          }
        }
      })
    },
    setColumnId (id) {
      this.columnId = id
      this.tree.currentNodeKey = id
      this.addForm.channelParentId = [id]
    },
    // 刷新左边栏目树
    reflushColumnTree (pid, name) {
      this.fetchChannelTree().then(res => {
        if (this.columnId) {
          this.setColumnId(this.columnId)
        }
      })
    },
    handleHeaderPreview () { },
    handleHeaderPermission () { },
    handleHeaderRelease () { },
    handleHeaderMerge () { },
    handleHeaderBatchCreate () { },
    handleHeaderDelete () { },
    handleConfirmStatic () { },
    handleConfirmMerge () { },
    handleConfirmBatch () { },
    getTheNodMerge () { },
    handleHeaderBrowse () { }
  },
  mounted () {
    const { id } = this.$route.query
    if (id) {
      this.setColumnId(Number(id))
    }
    this.FetchColumnOptions()
    this.FetchColumnModelOptions()
  },
  activated () {
    const { id } = this.$route.query
    if (id && id !== this.columnId) {
      this.setColumnId('')
      this.$nextTick(() => {
        this.setColumnId(Number(id))
      })
    }
  }
}
